function [Features,G,X_out,ErrList] = TDVM(IncompleteData, Omega, D,mu0,K,epsilon)
%  Feature Extraction for Incomplete Data via Low-rank Tucker Decomposition
% Let M be the number of tensors, let N be the order of each tensor
% Created by Qiquan SHI in 16 Nov. 2016
% Revised by Qiquan SHI in 25 Nov. 2016
% Revised by Qiquan SHI in 18 Jan. 2017
% Revised by Qiquan SHI in 3 April. 2017


FeaSize=size(IncompleteData);
TenSize=FeaSize(1:length(FeaSize)-1); % size of orgininal tensors

if nargin < 3
    D= min(TenSize); % size of tensors after dimension reduction
end
if nargin < 4
    mu0=1;
end
if nargin < 5
    K=500;
end
if nargin < 6
    epsilon=1e-30;
end
M=FeaSize(length(FeaSize));
N=length(FeaSize)-1;
alpha = ones(1, N);
alpha = alpha / sum(alpha);
rho = 10; % a larger rho makes the algorithm converges faster 
mu_max = 1e10;

T=cell(M,1);
X=cell(M,1);
U=cell(N,1);
G=cell(M,1);
Y = cell(M, N);
S= Y;

ErrList=zeros(M,K);
ErrList2=zeros(1,K);

for m=1:M
    if(length(FeaSize)==3)
        T{m,1}=IncompleteData(:,:,m);
        Omega_m0= IncompleteData(:,:,m)==1e-27;
        Omega_m=1-Omega_m0;
        Omega_m=logical(Omega_m);
        X{m,1}=IncompleteData(:,:,m);
        X{m,1}(logical(1-Omega_m)) = mean(T{m,1}(Omega_m));
        G{m,1}=rand(D);
        CoreSize=D;
        
    elseif(length(FeaSize)==4)
        T{m,1}=IncompleteData(:,:,:,m);
        Omega_m=Omega(:,:,:,m);
        X{m,1}=IncompleteData(:,:,:,m);
        X{m,1}(logical(1-Omega_m)) = mean(T{m,1}(Omega_m));
        G{m,1}=rand(D);
        CoreSize=D;
        
    else
        T{m,1}=IncompleteData(:,:,:,:,m);
        X{m,1}(logical(1-Omega)) = mean(T{m,1}(Omega));
        G{m,1}=rand(D);
        CoreSize=[D];
    end
    if m==1
        G_Mean=zeros(CoreSize);
        for n = 1 : N
            U{n} = rand(TenSize(n),D(n));
            temp=sqrt(sum(U{n}.^2,1));
            U{n}= U{n}./((ones(size(U{n},1),1))*temp);
            Y{m, n} = zeros(CoreSize);
            S{m, n} = zeros(CoreSize);
        end
    else
        for n = 1 : N
            Y{m, n} = zeros(CoreSize);
            S{m, n} = zeros(CoreSize);
        end
    end
    G_Mean=G_Mean+ G{m,1};
end
Features=zeros([M CoreSize ]);
X_out=zeros(FeaSize);

L=1;
for l=1:L
    for m=1:M
        midXtest = tensor(X{m,1});
        midTGtest = ttm(midXtest, U, 1: N, 't');
        G{m,1}=midTGtest.data;
        G_Meanmold=G_Mean - G{m,1};
        mu=mu0;
        for k=1:K
            
            mu1 = 1/mu;
            Dsum = zeros(CoreSize);
            Ysum = zeros(CoreSize);
            for ni=1:N
                S{m,ni} = Fold(Pro2TraceNorm(Unfold(G{m,1}+Y{m,ni}*mu1, CoreSize, ni), alpha(ni)*mu1), CoreSize, ni);
                Dsum = Dsum + S{m,ni};
                Ysum = Ysum + Y{m,ni};
            end
            % Update core tensor G
            midTG0 = tensor(X{m,1});
            midTG = ttm(midTG0, U, 1: N, 't');
            RefoldDY= mu*Dsum - Ysum;
            G{m,1} = (M^2 /(mu*M^2*N + 2*M-1))*(RefoldDY + midTG.data) - ((M-1) /(mu*M^2*N + 2*M-1)) * (G_Meanmold);
            
            % Update factor matrices U{n}
            for nii = 1: N
                midG = tensor(G{m,1});
                for nj = 1: N
                    if nj == nii
                        continue;
                    end
                    midG = ttm(midG, U{nj}, nj);
                end
                Gmn = unfold_mode_n_Nthorder(double(midG), nii, size(midG));
                Xmn = unfold_mode_n_Nthorder(X{m,1}, nii, TenSize);
                Temp_data = double(Xmn*Gmn');
                [UU, SS, DD] = svd(Temp_data,'econ');
                U{nii} = UU*DD';
            end
            clear Gmn;
            clear Xmn;
            
            midT = ttm(midG, U{N}, N);   
            X{m,1} = midT.data;
            
            X{m,1}(Omega_m) = T{m,1}(Omega_m);

      % Updating Y
            stopC = zeros(1,N);
            for in = 1 : N
                Y{m,in} = Y{m,in} + mu*(G{m,1} - S{m,in});
                stopC(in) = max(0, norm(G{m,1}(:) - S{m,in}(:))/norm(G{m,1}(:)));
            end
            ErrList(m,k) = max(stopC);
            
            if ErrList(m,k) < epsilon
                fprintf('ErrList: iterations = %d,  ith_image = %d, stopC=%f\n', k,m,ErrList(m,k));
                break;
            else
                mu = min(mu*rho, mu_max); %Update mu by stepsize rho
            end
        end
        G_Mean= G_Meanmold + G{m,1}; % Update mean of features (core tensors)
 
  %% Extracted features = {G_1,...,G_m..,G_M}
        if(length(FeaSize)==3)
           Features(m,:,:)=G{m,1};
           X_out(:,:,m)=X{m,1};
        elseif(length(FeaSize)==4)
           Features(m,:,:,:)=G{m,1};
           X_out(:,:,:,m)=X{m,1};
        else
           Features(m,:,:,:,:)=G{m,1};
           X_out(:,:,:,:,m)=X{m,1};
        end
 
    end
end




